import std.stdio;
import std.getopt;
import userinfo;
import message;
import cryptoclass;
import networker;

void generateAndSaveKeys(string write_path)
{
	UserInfo user;
	
	writeln("[*] Generating keys...");

	generateKeysForUser(user);
	writeln("[+] Generated user-keys:");
	writeln(">> Public-Key:  ", user.b64_publickey);
	writeln(">> Private-Key: ", user.b64_privatekey);
	
	writeln("[*] Saving keys to files...");
	saveKeysToFile(user, write_path);
	writeln("[+] Saved keys into files: ");
	writeln("Public-Key  -> ", write_path ~ ".pub");
	writeln("Private-Key -> ", write_path ~ ".priv");
}

UserInfo loadConfigs(string config_path)
{
	immutable string priv_key_file = config_path ~ ".priv";
	immutable string pub_key_file  = config_path ~ ".pub";

	// Load the users keys
	writeln("[*] Loading keys from ", config_path);
	UserInfo user = loadUserInfoFromFile(priv_key_file, pub_key_file);
	writeln("[+] Loaded keys:");
	writeln("\t->", priv_key_file);
	writeln("\t->", pub_key_file);

	return user;
}

void main(string[] args)
{
	string write_path;
	string config_path;
	string message;
	string b64_recipient_pubkey;
	string server_url;

	bool fetch = false;

	auto flags = getopt(args,
	std.getopt.config.bundling,
	"w|write", "Path to generate and write keys to. ( ./config/alice )", &write_path,
	"c|config", "File prefix to load key files from. ( ./configs/alice )", &config_path, 
	"m|messsage", "The message that should be sent. ( \"Hi bob how are you?\" ) ", &message,
	"f|fetch", "If this option is set the dropper only fetches messages for the public-key from -config", &fetch,
	"r|recipient", "Base64 encoded public-key of the recipient", &b64_recipient_pubkey,
	"u|url", "URL of the deaddrop server.", &server_url);

	if (write_path != "")
	{
		generateAndSaveKeys(write_path);
		return;
	}

	if (fetch && config_path != "" && server_url != "") 
	{
		UserInfo user = loadConfigs(config_path);
		Cryptoclass crypto = new Cryptoclass(user);
		Networker networker = new Networker(server_url);

		writeln("\n📧📧📧 You've got mail 📧📧📧\n");
		
		auto list = networker.getMessagesForRecipient(user.b64_publickey);
		foreach (single_message; list)
		{
			auto bin_msg = crypto.decryptMessage(single_message["payload"]);
			writeln(cast(string)bin_msg);
		}

		return;
	}
	else if (flags.helpWanted 
	|| config_path == ""
	|| message == ""
	|| b64_recipient_pubkey == ""
	|| server_url == "") 
	{
		defaultGetoptPrinter("Dropper to post and get messages from a deaddrop server.",
			flags.options);
		return;
	}

	// Load the configs
	UserInfo user = loadConfigs(config_path);

	// Create the crypto class
	Cryptoclass crypto = new Cryptoclass(user);
	Networker networker = new Networker(server_url);

	// Timestamp the message
	string timestamped_msg = prependTimeStamp(message);

	// Encrypt the message
	writeln("[*] Encrypting message...");
	auto binary_payload = crypto.encryptMessage(b64_recipient_pubkey, timestamped_msg);
	writeln("[*] Encoding payload...");
	string encoded_payload = encodePayload(binary_payload);

	// Format the message for a post request
	writeln("[*] Formatting message...");
	Message msg = formatMessage(b64_recipient_pubkey, encoded_payload);

	// Post message to server 
	writeln("[!] Posting message to server ", server_url);
	networker.postMessage(msg);
	writeln("[+] Done :^)");
}