module cryptoclass;

import userinfo : UserInfo;

/// This class handles the encryption and
/// decryption of messages to the `UserInfo` 
/// that was specified while creating this class
class Cryptoclass
{
private:
    import libsodium;
    import std.stdio : stderr;
    import std.base64 : Base64;
    import core.stdc.stdlib : exit, EXIT_FAILURE;


    UserInfo p_me;

public:

    /// This constructor sets the `UserInfo` for the crypto class 
    /// that is used to decrypt messages. It also makes sure
    /// that libsodium is loaded and initialized
    this(UserInfo user)
    {
        // Load libsodium
        if (sodium_init() < 0)
        {
            stderr.writeln("[-] Failed to load libsodium! Is it installed?");
            exit(EXIT_FAILURE);
        }

        this.p_me = user;
    }

    /// This function encrypts messages for the base64 encoded public
    /// key that is passed to the function and returns a ubyte array
    /// containing the payload
    ubyte[] encryptMessage(string b64_recipientPublicKey, string message)
    {
        import std.conv : to;

        ubyte[] messageBin = cast(ubyte[])message;
        ubyte[] cipherText = new ubyte[(crypto_box_SEALBYTES + messageBin.length)];
        ubyte[] recipientPublicKey = Base64.decode(b64_recipientPublicKey);

        crypto_box_seal(cipherText.ptr, messageBin.ptr, messageBin.length, recipientPublicKey.ptr);

        return cipherText;
    }

    /// This function decrypts a given base64 encoded
    /// payload string using the internal userinfos
    /// public and private key and returns 
    /// the decrypted buffer as a ubyte array
    ubyte[] decryptMessage(string messagePayload)
    {
        ubyte[] payloadBinary = Base64.decode(messagePayload);
        ubyte[] decrypted = new ubyte[payloadBinary.length];

        ubyte[crypto_box_PUBLICKEYBYTES] pubkey = p_me.getPublicKey();
        ubyte[crypto_box_SECRETKEYBYTES] seckey = p_me.getPrivateKey(); 

        if (crypto_box_seal_open(
            decrypted.ptr, 
            payloadBinary.ptr, 
            payloadBinary.length, 
            pubkey.ptr, 
            seckey.ptr
        ) != 0 ) {
            stderr.writeln("[!] Failed to open sealed message!!!");
        }

        return decrypted;
    }
}