module userinfo;

import std.base64 : Base64;

struct UserInfo
{
public:
    string b64_publickey;
    string b64_privatekey;

    ubyte[] getPublicKey()
    {
        return Base64.decode(b64_publickey);
    }

    ubyte[] getPrivateKey()
    {
        return Base64.decode(b64_privatekey);
    }
}

void generateKeysForUser(out UserInfo user)
{
    import libsodium : crypto_box_keypair, crypto_box_PUBLICKEYBYTES, crypto_box_SECRETKEYBYTES;
    import std.stdio : stderr;

    ubyte[crypto_box_PUBLICKEYBYTES] publicKey;
    ubyte[crypto_box_SECRETKEYBYTES] privateKey;

    if (crypto_box_keypair(publicKey.ptr, privateKey.ptr) != 0) 
    {
        stderr.writeln("[-] Failed to generate keypair!");
    }

    user.b64_publickey  = Base64.encode(publicKey);
    user.b64_privatekey = Base64.encode(privateKey); 
}

UserInfo loadUserInfoFromFile(string privateKeyFile, string publicKeyFile)
{
    import std.file : readText;

    UserInfo user;
    
    user.b64_publickey  = readText(publicKeyFile);
    user.b64_privatekey = readText(privateKeyFile);

    return user;
}

void saveKeysToFile(UserInfo user, string file_name_prefix)
{
    import std.file : write;

    write(file_name_prefix ~ ".pub", user.b64_publickey);
    write(file_name_prefix ~ ".priv", user.b64_privatekey);
}

UserInfo createRecipient(string b64_publicKey)
{
    UserInfo recipient;
    recipient.b64_publickey  = b64_publicKey;
    recipient.b64_privatekey = "";

    return recipient;
}